// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.jboss

import java.util.StringTokenizer;

import javax.xml.parsers.DocumentBuilderFactory
import javax.xml.parsers.DocumentBuilder
import javax.xml.transform.dom.DOMSource
import javax.xml.transform.stream.StreamResult
import javax.xml.transform.TransformerFactory
import javax.xml.transform.Transformer
import javax.xml.transform.OutputKeys
import javax.xml.xpath.XPathFactory
import javax.xml.xpath.XPathConstants
import org.w3c.dom.Document
import org.w3c.dom.Node
import java.util.jar.JarFile
//import org.apache.commons.io.FileUtils

class Datasource47 extends Resource {

	String resourceType = "datasource"
	String name;
	String jndiname;
	String servername;
	String classpath;
	String driver;
	String xa;
	String connectionUrl;
	String user;
	String password;
	String encoding;
	String driverOptions;
	String hostName;
	String serverPort;
	//String dataSourceClass;
	//String xaDataSourceClass;

	String max_pool_size; //max-pool-size
	String min_pool_size;   //min-pool-size
	String prefill;
	String use_strict_min; //for use-strict-min
	String flush_strategy; //flush-strategy

	//for xa only
	String is_same_rm_override ; //is-same-rm-override
	String interleaving ;
	String no_tx_separate_pools; //no-tx-separate-pools
	String pad_xid ; //pad-xid
	String wrap_xa_resource ; //wrap-xa-resource

	//for security domain
	String securityDomain ;

	//for validation
	String check_valid_connection_sql ; //check-valid-connection-sql
	String validate_on_match ;//validate-on-match
	String background_validation ;//background-validation
	String background_validation_millis ;//background-validation-millis
	String use_fast_fail; //use-fast-fail

	//for time out
	String blocking_timeout_millis ; //blocking-timeout-millis
	String idle_timeout_minutes ; //idle-timeout-minutes
	String set_tx_query_timeout ; //set-tx-query-timeout
	String query_timeout ; //query-timeout
	String use_try_lock ;//use-try-lock
	String allocation_retry ; //allocation-retry
	String allocation_retry_wait_millis ; //allocation-retry-wait-millis
	String xa_resource_timeout; //xa-resource-timeout

	//for statement
	String track_statements //track-statements
	String prepared_statement_cache_size //prepared-statement-cache-size
	String share_prepared_statements //share-prepared-statements

	//for recover
	String recover_credential_user ; // recover-credential user recover-credential-user
	String recover_credential_password ; // recover-credential password recover-credential-password
	String recover_credential_security_domain ; // recover-credential user recover-credential-security-domain

	boolean usePool;
	boolean useSecurityDomain;
	boolean useValidation;
	boolean useTimeOut ;
	boolean useStatement ;
	boolean useRecover ;

	//reauth-plugin
	ArrayList<String> reauth_plugin_config_properties = new ArrayList<String>() ;
	String reauth_plugin_class_name ;

	//valid-connection-checker
	ArrayList<String> valid_connection_checker_config_properties = new ArrayList<String>() ;
	String valid_connection_checker_class_name ;

	//stale-connection-checker
	ArrayList<String> stale_connection_checker_config_properties = new ArrayList<String>() ;
	String stale_connection_checker_class_name ;

	//exception-sorter
	ArrayList<String> exception_sorter_config_properties = new ArrayList<String>() ;
	String exception_sorter_class_name ;

	//recover-plugin
	ArrayList recover_plugin_config_properties = new ArrayList<String>() ;
	String recover_plugin_class_name ;

	Collection<DatasourceOption> dumpedOptions

	Datasource47() {}
	Datasource47(String n,String j, String s, String c, String d, String x,String url, String u, String p, String o, String host, String port){
		name = n
		jndiname = j
		servername = s
		classpath = c
		driver = d
		xa = x
		connectionUrl = url
		user = u
		password = p
		driverOptions = o
		hostName = host
		serverPort = port

		dumpedOptions = new ArrayList<DatasourceOption>();

		for (option in driverOptions.tokenize(',')){

			Logger.log('TRACE', "datasource - looping over options: option = $option")
			def namevalue = option.split("=")
			Logger.log('TRACE', "datasource - looping over options: namevalue = $namevalue")
			def optionName = namevalue[0]
			Logger.log('TRACE', "datasource - looping over options: optionName = $optionName")
			def value = ""

			value = option - (optionName+"=")

			Logger.log('DEBUG', "datasource - processing options: optionName=$optionName value=$value");
			if (
			! optionName.compareTo("blocking-timeout-millis") ||
			! optionName.compareTo("check-valid-connection-sql") ||
			! optionName.compareTo("exception-sorter-class-name") ||
			! optionName.compareTo("idle-timeout-minutes") ||
			! optionName.compareTo("isSameRM-override-value") ||
			! optionName.compareTo("max-pool-size") ||
			! optionName.compareTo("min-pool-size") ||
			! optionName.compareTo("new-connection-sql") ||
			! optionName.compareTo("no-tx-separate-pools") ||
			! optionName.compareTo("prepared-statement-cache-size") ||
			! optionName.compareTo("share-prepared-statements") ||
			! optionName.compareTo("track-connection-by-tx") ||
			! optionName.compareTo("track-statements") ||
			! optionName.compareTo("transaction-isolation") ||
			! optionName.compareTo("use-java-context") ||
			! optionName.compareTo("valid-connection-checker-class-name") ||
			! optionName.compareTo("xa-datasource-class")
			) {
				dumpedOptions.add(new DatasourceOption(optionName,value));
			}
			/*
			 else if (! optionName.compareTo("type-mapping") ){
			 def metadata = datasourceParser.addNode("metadata","",tx_datasource)
			 datasourceParser.addNode(optionName,value,metadata)
			 }
			 */
			else if ( ! optionName.compareTo("security-domain")){

				dumpedOptions.add(new DatasourceOption("security-domain",value));
			}

			else {
				def tx_datasource_property
				if (xa == "true") {
					tx_datasource_property = "xa-datasource-property"
				} else {
					tx_datasource_property = "connection-property"
				}
				dumpedOptions.add(new DatasourceOption(optionName, value, tx_datasource_property))
			}

		}
		if (user!=null && user!="not-set"){
			dumpedOptions.add(new DatasourceOption("user-name", user))
			dumpedOptions.add(new DatasourceOption("password", "xxxxxxxx"))
		}


	}

	int configure(String instance, Properties scriptProperties){
		try {

			Logger.log('DEBUG', "datasource - Configuring ${instance}")

			name = scriptProperties.get("datasource."+instance+".name")
			jndiname = scriptProperties.get("datasource."+instance+".jndiname")
			servername = scriptProperties.get("datasource."+instance+".servername")
			classpath = scriptProperties.get("datasource."+instance+".classpath")
			driver = scriptProperties.get("datasource."+instance+".driver")
			xa = scriptProperties.get("datasource."+instance+".xa")
			connectionUrl = scriptProperties.get("datasource."+instance+".connectionUrl")
			user = scriptProperties.get("datasource."+instance+".user")
			password = scriptProperties.get("datasource."+instance+".password")
			encoding = scriptProperties.get("datasource."+instance+".encoding")
			driverOptions = scriptProperties.get("datasource."+instance+".options")
			hostName = scriptProperties.get("datasource."+instance+".hostName")
			serverPort = scriptProperties.get("datasource."+instance+".serverPort")
			//dataSourceClass = scriptProperties.get("datasource."+instance+".dataSourceClass")
			//xaDataSourceClass = scriptProperties.get("datasource."+instance+".xaDataSourceClass")
			//def jbossDatasourceOptions = scriptProperties.get("datasource."+instance+".jbossDatasourceOptions")
			def jbossInstallDir = scriptProperties.'config.jboss.install.dir'

			Logger.log('DEBUG',"datasource - datasourcename=${name}")
			Logger.log('DEBUG',"datasource - jndiname=${jndiname}")
			Logger.log('DEBUG',"datasource - servername=${servername}")
			Logger.log('DEBUG',"datasource - classpath=${classpath}")
			Logger.log('DEBUG',"datasource - driver=${driver}")
			Logger.log('DEBUG',"datasource - xa=${xa}")
			Logger.log('DEBUG',"datasource - connectionUrl=${connectionUrl}")
			Logger.log('DEBUG',"datasource - user=${user}")
			Logger.log('DEBUG',"datasource - password=xxxxxx")
			Logger.log('DEBUG',"datasource - driverOptions=${driverOptions}")
			Logger.log('DEBUG',"datasource - hostName=${hostName}")
			Logger.log('DEBUG',"datasource - serverPort=${serverPort}")
			//Logger.log('DEBUG',"datasource - jbossDatasourceOptions=${jbossDatasourceOptions}")
			Logger.log('DEBUG',"datasource - jbossInstallDir=${jbossInstallDir}")

			String driverModuleName = createModules(classpath,name,jbossInstallDir)
			def datasourceFile = new String("${jbossInstallDir}/standalone/configuration/${servername}.xml")

			//		def datasourceFile = new String("${jbossInstallDir}/server/${servername}/deploy/${name}-ds.xml")

			//Don't have commons IO, so using ant...
			//FileUtils.touch( new File( datasourceFile ) )
			//		Ant.touch(datasourceFile)
			//String driverModuleName = "com.sas.drivers."+name
			def datasourceParser = new Parser(datasourceFile)
			def datasourcesNode = datasourceParser.findNode("//datasources")
			def driversNode = datasourceParser.findNode("//drivers")
			def driverNode = datasourceParser.addNode("driver","",driversNode)
			datasourceParser.addAttribute("name", name+"_driver", driverNode)
			datasourceParser.addAttribute("module", driverModuleName, driverNode)
			//def datasourceClassNode = datasourceParser.addNode("datasource-class",driver,driverNode)
			def datasourceClassNode = datasourceParser.addNode("driver-class",driver,driverNode)
/*			if(dataSourceClass!=null){
			  def datasourceClassNode = datasourceParser.addNode("datasource-class",dataSourceClass,driverNode)
			}
			if(xaDataSourceClass!=null){
				def datasourceClassNode = datasourceParser.addNode("xaDatasource-class",xaDataSourceClass,driverNode)
			}*/
  
			//		def tx_datasource
			//		def _datasource
			
			def datasourceNode
			def securityNode
			def userNameNode
			def passwordNode
			def connectionURLNode
			if (xa == "true") {
				datasourceNode = datasourceParser.addNode("xa-datasource","",datasourcesNode)
				//add xa-datasource-class
				//datasourceParser.addNode("xa-datasource-class",driver,datasourceNode)
				def xaDatasourcePropertyNode =  datasourceParser.addNode("xa-datasource-property",connectionUrl,datasourceNode)
				datasourceParser.addAttribute("name", "URL", xaDatasourcePropertyNode)
				datasourceParser.addNode("driver",name+"_driver",datasourceNode)
				datasourceParser.addAttribute("jta", "true", datasourceNode)
				datasourceParser.addAttribute("enabled", "true", datasourceNode)
				datasourceParser.addAttribute("use-java-context", "true", datasourceNode)
				datasourceParser.addAttribute("pool-name", name+"_pool", datasourceNode)

			} else {
				datasourceNode = datasourceParser.addNode("datasource","",driversNode,"Sibling")
				//datasourceParser.addNode("driver-class",driver,datasourceNode)
				datasourceParser.addNode("connection-url",connectionUrl, datasourceNode)
				datasourceParser.addNode("driver",name+"_driver",datasourceNode)
				datasourceParser.addAttribute("jta", "true", datasourceNode)
				datasourceParser.addAttribute("enabled", "true", datasourceNode)
				datasourceParser.addAttribute("use-java-context", "true", datasourceNode)
				datasourceParser.addAttribute("pool-name", name+"_pool", datasourceNode)
			}

			securityNode = datasourceParser.addNode("security", "", datasourceNode)


			String options=driverOptions

			Logger.log('DEBUG', "datasource - options = $options")
			for (option in options.tokenize(',')){
				Logger.log('TRACE', "datasource - looping over options: option = $option")
				def namevalue = option.split("=")
				Logger.log('TRACE', "datasource - looping over options: namevalue = $namevalue")
				def optionName = namevalue[0]
				Logger.log('TRACE', "datasource - looping over options: optionName = $optionName")
				def value = ""

				value = option - (optionName+"=")

				Logger.log('DEBUG', "datasource - processing options: optionName=$optionName value=$value");
				if (
				! optionName.compareTo("new-connection-sql") ||
				! optionName.compareTo("transaction-isolation") ||
				! optionName.compareTo("xa-datasource-class") ||
				! optionName.compareTo("url-delimiter") ||
				! optionName.compareTo("url-selector-strategy-class-name")
				) {
					datasourceParser.addNode(optionName,value,datasourceNode)
				}
				else if (! optionName.compareTo("type-mapping") ){
					def metadata = datasourceParser.addNode("metadata","",tx_datasource)
					datasourceParser.addNode(optionName,value,metadata)
				}
				else if ( ! optionName.compareTo("security-domain")){
					//user = null //Don't use username or password elements
					//datasourceParser.addNode(optionName,value,tx_datasource)
					securityDomain = value
					useSecurityDomain = true
				}else if( ! optionName.compareTo("maxPoolSize")){
					maxPoolSize = value
					usePool = true
				}else if( ! optionName.compareTo("minPoolSize")){
					minPoolSize = value
					usePool = true
				}else if( ! optionName.compareTo("prefill")){
					prefill = value
					usePool = true
				}else if( ! optionName.compareTo("use-strict-min")){
					use_strict_min = value
					usePool = true
				}else if( ! optionName.compareTo("flush-strategy")){
					flush_strategy = value
					usePool = true
				}else if( ! optionName.compareTo("is-same-rm-override")){
					is_same_rm_override = value
					usePool = true
				}else if( ! optionName.compareTo("interleaving")){
					interleaving= value
					usePool = true
				}else if( ! optionName.compareTo("no-tx-separate-pools")){
					no_tx_separate_pools= value
					usePool = true
				}else if( ! optionName.compareTo("pad-xid")){
					pad_xid= value
					usePool = true
				}else if( ! optionName.compareTo("wrap-xa-resource")){
					wrap_xa_resource= value
					usePool = true
				}else if(optionName.indexOf("reauth-plugin")>0){
					if(! optionName.compareTo("reauth-plugin-class-name")){
						reauth_plugin_class_name = value
					}
					reauth_plugin_config_properties.add(option)
					usePool = true
				}else if(optionName.indexOf("valid-connection-checker")>0){
					if(! optionName.compareTo("valid-connection-checker-class-name")){
						valid_connection_checker_class_name = value
					}
					valid_connection_checker_config_properties.add(option)
					useValidation = true
				}else if(optionName.indexOf("stale-connection-checker")>0){
					if(! optionName.compareTo("stale-connection-checker-class-name")){
						stale_connection_checker_class_name = value
					}
					stale_connection_checker_config_properties.add(option)
					useValidation = true
				}else if(optionName.indexOf("exception-sorter")>0){
					if(! optionName.compareTo("exception-sorter-class-name")){
						exception_sorter_class_name = value
					}
					exception_sorter_config_properties.add(option)
					useValidation = true
				}else if(optionName.indexOf("recover-plugin")>0){
					if(! optionName.compareTo("recover-plugin-class-name")){
						recover_plugin_class_name = value
					}
					recover_plugin_config_properties.add(option)
					useRecover = true
				}else if(! optionName.compareTo("check-valid-connection-sql")){
					check_valid_connection_sql = value
					useValidation = true
				}else if(! optionName.compareTo("validate-on-match")){
					validate_on_match = value
					useValidation = true
				}else if(! optionName.compareTo("background-validation")){
					background_validation = value
					useValidation = true
				}else if(! optionName.compareTo("background-validation-millis")){
					background_validation_millis = value
					useValidation = true
				}else if(! optionName.compareTo("use-fast-fail")){
					use_fast_fail = value
					useValidation = true
				}else if(! optionName.compareTo("blocking-timeout-millis")){
					blocking_timeout_millis = value
					useTimeOut = true
				}else if(! optionName.compareTo("idle-timeout-minutes")){
					idle_timeout_minutes = value
					useTimeOut = true
				}else if(! optionName.compareTo("tx-query-timeout")){
					tx_query_timeout = value
					useTimeOut = true
				}else if(! optionName.compareTo("query-timeout")){
					query_timeout = value
					useTimeOut = true
				}else if(! optionName.compareTo("use-try-lock")){
					use_try_lock = value
					useTimeOut = true
				}else if(! optionName.compareTo("allocation-retry")){
					allocation_retry = value
					useTimeOut = true
				}else if(! optionName.compareTo("allocation-retry-wait-millis")){
					allocation_retry_wait_millis = value
					useTimeOut = true
				}else if(! optionName.compareTo("xa-resource-timeout")){
					xa_resource_timeout = value
					useTimeOut = true
				}else if(! optionName.compareTo("track-statements")){
					track_statements = value
					useStatement = true
				}else if(! optionName.compareTo("prepared-statement-cache-size")){
					prepared_statement_cache_size = value
					useStatement = true
				}else if(! optionName.compareTo("share-prepared-statements")){
					share_prepared_statements = value
					useStatement = true
				}else if(! optionName.compareTo("recover-credential-user")){
					recover_credential_user = value
					useRecover = true
				}else if(! optionName.compareTo("recover-credential-password")){
					recover_credential_password = value
					useRecover = true
				}else if(! optionName.compareTo("recover-credential-security-domain")){
					recover_credential_security_domain = value
					useRecover = true
				}
				else {
					def datasource_property
					if(value.equals("(DSN=SharedServices;encoding=UNICODE_FSS)")){
						value = "(DSN=SharedServices)";
					}
					if (xa == "true") {
						datasource_property = datasourceParser.addNode("xa-datasource-property",value,datasourceNode)
					} else {
						datasource_property = datasourceParser.addNode("connection-property",value,datasourceNode)
					}
					datasourceParser.addAttribute("name",optionName,datasource_property)
				}
			}
/*
			if(usePool){
				def poolNode


				if (xa == "true") {
					poolNode = datasourceParser.addNode("xa-pool", "", datasourcesNode)
				}
				else{
					poolNode = datasourceParser.addNode("pool", "", datasourcesNode)
				}
				createNode(datasourceParser,poolNode,"max-pool-size",max_pool_size)
				createNode(datasourceParser,poolNode,"min-pool-size",min_pool_size)
				createNode(datasourceParser,poolNode,"prefill",prefill)
				createNode(datasourceParser,poolNode,"use-strict-min",use_strict_min)
				createNode(datasourceParser,poolNode,"flush-strategy",flush_strategy)

				if (xa == "true") {
					createNode(datasourceParser,poolNode,"is-same-rm-override",is_same_rm_override)
					createNode(datasourceParser,poolNode,"interleaving",interleaving)
					createNode(datasourceParser,poolNode,"no-tx-separate-pools",no_tx_separate_pools)
					createNode(datasourceParser,poolNode,"pad-xid",pad_xid)
					createNode(datasourceParser,poolNode,"wrap-xa-resource",wrap_xa_resource)
				}
				createExtensionTypeNode(datasourceParser,poolNode,"reauth-pluginr",reauth_plugin_config_properties,reauth_plugin_class_name)
			}
			if(useValidation){
				def validationNode
				validationNode = datasourceParser.addNode("validation", "", datasourcesNode)
				createNode(datasourceParser,validationNode,"check-valid-connection-sql",check_valid_connection_sql)
				createNode(datasourceParser,validationNode,"validate-on-match",validate_on_match)
				createNode(datasourceParser,validationNode,"background-validation",background_validation)
				createNode(datasourceParser,validationNode,"background-validation-millis",background_validation_millis)
				createNode(datasourceParser,validationNode,"use-fast-fail",use_fast_fail)
				createExtensionTypeNode(datasourceParser,validationNode,"valid-connection-checker",valid_connection_checker_config_properties,valid_connection_checker_class_name)
				createExtensionTypeNode(datasourceParser,validationNode,"stale-connection-checker",stale_connection_checker_config_properties,stale_connection_checker_class_name)
				createExtensionTypeNode(datasourceParser,validationNode,"exception-sorter",exception_sorter_config_properties,exception_sorter_class_name)
			}
			if(useTimeOut){
				def timeoutNode
				timeoutNode = datasourceParser.addNode("timeout", "", datasourcesNode)
				createNode(datasourceParser,timeoutNode,"blocking-timeout-millis",blocking_timeout_millis)
				createNode(datasourceParser,timeoutNode,"idle-timeout-minutes",idle_timeout_minutes)
				createNode(datasourceParser,timeoutNode,"set-tx-query-timeout",set_tx_query_timeout)
				createNode(datasourceParser,timeoutNode,"query-timeout",query_timeout)
				createNode(datasourceParser,timeoutNode,"use-try-lock",use_try_lock)
				createNode(datasourceParser,timeoutNode,"allocation-retry",allocation_retry)
				createNode(datasourceParser,timeoutNode,"allocation-retry-wait-millis",allocation_retry_wait_millis)
				createNode(datasourceParser,timeoutNode,"xa-resource-timeout",xa_resource_timeout)
			}
			if(useStatement){
				def statementsNode
				statementsNode = datasourceParser.addNode("statement", "", datasourcesNode)
				createNode(datasourceParser,statementsNode,"track-statements",track_statements)
				createNode(datasourceParser,statementsNode,"prepared-statement-cache-size",prepared_statement_cache_size)
				createNode(datasourceParser,statementsNode,"share-prepared-statements",share_prepared_statements)
			}
			if(useRecover){
				def recoverNode
				def recoverCredentialNode
				recoverNode = datasourceParser.addNode("recover", "", datasourcesNode)
				recoverCredentialNode = datasourceParser.addNode("recover-credential","",recoverNode)
				createNode(datasourceParser,recoverCredentialNode,"user-name",recover_credential_user)
				createNode(datasourceParser,recoverCredentialNode,"password",recover_credential_password)
				createNode(datasourceParser,recoverCredentialNode,"security-domain",recover_credential_security_domain)
				createExtensionTypeNode(datasourceParser,recoverNode,"recover-plugin",recover_plugin_config_properties,recover_plugin_class_name)
			}
			*/
			if (jndiname!=null){
				datasourceParser.addAttribute("jndi-name", "java:jboss/"+jndiname, datasourceNode)
			}

			if(useSecurityDomain){
					datasourceParser.addNode("security-domain", securityDomain, securityNode)
			}
			
			if (user!=null && (!user.equals("not-set"))){
				datasourceParser.addNode("user-name", user, securityNode)
				String prompt = "\nEnter Password \n\tDatasource=$name \n\tuser=$user \n\tPassword="
				password = com.sas.config.jboss.Utils.encodeDecodePassword(password, encoding, prompt)
				datasourceParser.addNode("password", password, securityNode)
		    }

			datasourceParser.output(datasourceFile)
			
			//datasourceParser.

			
		}
		catch (Throwable e) {
			Logger.log('DEBUG', "Failure : " + e.toString());
			e.printStackTrace();
			return 8;
		}
		return 0;
	}

	String createModules(String classpath,String modulesDir,String jbossInstallDir){
		String moduleName = null ;
		String mainModuleName = null ;
		for (String sourceJar in classpath.tokenize(';')){
			String jarFileName = searchForRootJar(sourceJar)
			moduleName = jarFileName.substring(0,jarFileName.indexOf(".jar"))
			moduleName = moduleName.replaceAll("\\.","-")
			createModule(sourceJar,modulesDir,moduleName,jbossInstallDir,classpath)
			//def targetDir = new String("${jbossInstallDir}/modules/com/sas/drivers/${modulesDir}/${moduleName}/main")
			JarFile jarFile = new JarFile(sourceJar)
			if(jarFile.getEntry(driver.replaceAll("\\.","/")+".class")!=null){
				//System.out.println(jarFile.getEntry(driver+".class"));
				mainModuleName = "com.sas.drivers."+modulesDir+"."+moduleName
				//System.out.println(mainModuleName)
			}
		}
		return mainModuleName
	}
	void createModule(String sourceJar,String modulesDir,String moduleNme,String jbossInstallDir,String wholeClassPath){
		def targetDir = new String("${jbossInstallDir}/modules/com/sas/drivers/${modulesDir}/${moduleNme}/main")
		def content = ""
		def dependencyContent = ""
        File targetDirFile = new File(targetDir)
		targetDirFile.mkdirs()
		
		Logger.log('DEBUG', "datasource - copy $sourceJar $targetDir")
		Ant.copyFileToDir(sourceJar,targetDir, "true")
		String jarFileName = searchForRootJar(sourceJar)
		content = content+"<resource-root path=\""+jarFileName+"\"/>"+System.getProperty("line.separator")
		
		wholeClassPath = wholeClassPath-sourceJar
		wholeClassPath = wholeClassPath.replaceAll(";;",";")
		for(String moduleJar in wholeClassPath.tokenize(';')){
			jarFileName = searchForRootJar(moduleJar)
			jarFileName = jarFileName.substring(0,jarFileName.indexOf(".jar"))
			moduleJar = jarFileName.replaceAll("\\.","-")
			dependencyContent = dependencyContent+"<module name=\"com.sas.drivers."+modulesDir+"."+moduleJar+"\"/>"+System.getProperty("line.separator")+"        "
		}
		def moduleWholeName = "com.sas.drivers."+modulesDir+"."+moduleNme;
        def moduleFileContent='''<?xml version="1.0" encoding="UTF-8"?>

<!--
  ~ JBoss, Home of Professional Open Source.
  ~ Copyright 2010, Red Hat, Inc., and individual contributors
  ~ as indicated by the @author tags. See the copyright.txt file in the
  ~ distribution for a full listing of individual contributors.
  ~
  ~ This is free software; you can redistribute it and/or modify it
  ~ under the terms of the GNU Lesser General Public License as
  ~ published by the Free Software Foundation; either version 2.1 of
  ~ the License, or (at your option) any later version.
  ~
  ~ This software is distributed in the hope that it will be useful,
  ~ but WITHOUT ANY WARRANTY; without even the implied warranty of
  ~ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  ~ Lesser General Public License for more details.
  ~
  ~ You should have received a copy of the GNU Lesser General Public
  ~ License along with this software; if not, write to the Free
  ~ Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  ~ 02110-1301 USA, or see the FSF site: http://www.fsf.org.
  -->
<module xmlns="urn:jboss:module:1.1" name="$moduleWholeName">

    <resources>
        $content
        <!-- Insert resources here -->
    </resources>
    <dependencies>
        <module name="javax.api"/>
        <module name="javax.transaction.api"/>
        <module name="javax.servlet.api" optional="true"/>
        <module name="org.omg.api" slot="main" /> 
        <module name="org.apache.log4j" slot="main" />
        $dependencyContent
    </dependencies>
</module>'''
		def engine = new groovy.text.SimpleTemplateEngine()
		def template = engine.createTemplate(moduleFileContent)
		def binding = [
			moduleWholeName : moduleWholeName,
			content : content,
			dependencyContent:dependencyContent,
			  ]
		def scriptOut = template.make(binding)
		def moduleFile = new File(targetDir+"/module.xml")
		moduleFile.write(scriptOut.toString())
 
		    
	}

	String searchForRootJar(String jarLocation){
		StringTokenizer st = new StringTokenizer(jarLocation,"\\/")
		String jarFileName = ""
		while(st.hasMoreElements()){
			jarFileName = (String)st.nextElement();
		}
        return jarFileName
	}

	void createNode(Parser parser,Node parentNode,String node,String value){
		if(value!=null){
			//Parser parser = new Parser(file)
			//parser.findNode("//")
			parser.addNode(node, value, parentNode)
		}
	}
    public static void main(String[] args){
		Properties pro = new Properties();
		pro.load(new FileInputStream(new File("C:\\SAS\\EBIserver\\Lev1\\Web\\Scripts\\JBoss\\jbossScripting.properties")));
		Datasource47 d = new Datasource47();
		d.configure("datasource1", pro)
		//String x = "lll.kkk.jjj"
		//x = x.replaceAll("\\.", "a")
		//System.out.println(x);
		//JarFile jf = new JarFile("C:\\SAS\\EBIserver\\Lev1\\Web\\Applications\\SASWIPServices9.3\\JDBCDrivers\\sas.oda.tkts.jar")
		//System.out.println(jf.getEntry("com/sas/tkts/TKTSDriver.class"));
		}

}

