// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.jboss

import javax.xml.parsers.DocumentBuilderFactory
import javax.xml.parsers.DocumentBuilder
import javax.xml.transform.dom.DOMSource
import javax.xml.transform.stream.StreamResult
import javax.xml.transform.TransformerFactory
import javax.xml.transform.Transformer
import javax.xml.transform.OutputKeys
import javax.xml.xpath.XPathFactory
import javax.xml.xpath.XPathConstants
import org.w3c.dom.Document
import org.w3c.dom.Node
//import org.apache.commons.io.FileUtils

class Loginmodule47 extends Resource {
	
	String resourceType = "loginmodule"
	String policy;
    String classname;
    String flag;
    String servername;
    String options;
	String username;
	String password;
	String encoding;
    Map optionKeys;
    Map optionMap;
    String deleted;
    //String name;
    
    Loginmodule47() {}
    Loginmodule47(String p, String c, String f,String s, String o, String u, String pass, String d){
        policy = p
        classname = c
        flag = f
        servername = s
        options = o
		username = u
		password = pass
        deleted = d

        optionMap = [:]
        optionKeys = [:]
        
        if (options) {
            def list = options.tokenize(",").collect { it as String }
            list.each { option ->
                def optionArray = option.split("=")
                def name=optionArray[0]
                def value
                if (optionArray.size()>2){
                    value=optionArray[1]+"="+optionArray[2]
                } else {
                    value=optionArray[1]
                }
                optionKeys[name]=name
                Logger.log('DEBUG',"loginmodule - processing name="+name)
                //Logger.log('DEBUG',"loginmodule - processing value="+value)
                
                if ((! name.compareTo("password"))&&(!classname.compareTo("org.jboss.resource.security.SecureIdentityLoginModule")) ){                 
                    def encodedPW
                    try{
                        def unencodedPW = com.sas.util.SasPasswordString.decode(value);
                        Logger.log('DEBUG',"loginmodule - processing password")
						ClassLoader classLoader = this.class.getClassLoader();
						Class aSecureIdentityLoginModule = classLoader.loadClass("org.jboss.resource.security.SecureIdentityLoginModule");
                        encodedPW = aSecureIdentityLoginModule.encode(unencodedPW);
                        value = encodedPW
                        Logger.log('DEBUG',"loginmodule - changing value="+value)
                    }
					catch (ClassNotFoundException cfe){
						encodedPW="PASSWORD"
						Logger.log('INFO',"loginmodule - can''t load SecureIdentityLoginModule to encode passwords,")
						Logger.log('INFO',"loginmodule - will use 'PASSWORD' instead.")						
						value = encodedPW
					}
                    catch(Throwable t){
                        encodedPW = "PASSWORD"
                        Logger.log('INFO',"loginmodule - Unable to encode passwords, will use 'PASSWORD' instead.")
                        t.printStackTrace();
                        value = encodedPW
                    }
                }
                optionMap[name]=value
            }
			if (username!=null && username!="not-set"){
				if (classname.contains("org.jboss.resource.security.SecureIdentityLoginModule")){
					optionKeys["username"]="username"
					optionMap["username"]=username
					
					optionKeys["password"]="password"
					optionMap["password"]="xxxxxxxx"
				} else if (classname.contains("com.sas.services.security.login")){
					optionKeys["trusteduser"]="trusteduser"
					optionMap["trusteduser"]=username
					
					optionKeys["trustedpw"]="trustedpw"
					optionMap["trustedpw"]="xxxxxxxx"

				}
			}

        }

        //Logger.log('DEBUG',"jbossScripting: optionList = " + optionMap.toString())
    }

    int configure(String instance, Properties scriptProperties){
        try {

    	Logger.log('DEBUG', "loginmodule - Configuring ${instance}")
        policy = scriptProperties.get("loginmodule."+instance+".policy")
        classname = scriptProperties.get("loginmodule."+instance+".classname")  
        flag = scriptProperties.get("loginmodule."+instance+".flag") 
        servername = scriptProperties.get("loginmodule."+instance+".servername")  
        options = scriptProperties.get("loginmodule."+instance+".options")
		username = scriptProperties.get("loginmodule."+instance+".user") 
		password = scriptProperties.get("loginmodule."+instance+".password") 
		encoding = scriptProperties.get("loginmodule."+instance+".encoding") 
        deleted = scriptProperties.get("loginmodule."+instance+".deleted")
        //name = classname
        
		if(classname!=null&&classname.equals("org.jboss.resource.security.SecureIdentityLoginModule")){
			classname = "org.picketbox.datasource.security.SecureIdentityLoginModule"
		}
        def jbossInstallDir = scriptProperties.'config.jboss.install.dir'
            
        //def loginmoduleFile = new String("$jbossInstallDir/server/$servername/conf/login-config.xml")
		def loginmoduleFile = new String("$jbossInstallDir/standalone/configuration/${servername}.xml")
        def loginmoduleParser = new Parser(loginmoduleFile)     
        def oldModuleNode=null
        def authenticationNode
        def loginmoduleNode
        def policyNode
        def applicationpolicyNode

                 
        if (options == null) options="" // we want to tokenize options, need a string later...
        
        //First find existing application policy...
        //policyNode = loginmoduleParser.findNode("//policy")
		policyNode = loginmoduleParser.findNode("//security-domains")
        applicationpolicyNode = loginmoduleParser.findNode("//security-domain[@name='${policy}']")
        if (applicationpolicyNode != null) {
            Logger.log('DEBUG',"Found Application Policy :${policy}")
            oldModuleNode = loginmoduleParser.findNode("//security-domain[@name='${policy}']/authentication/login-module[@code='${classname}']")
            if (oldModuleNode != null){
                //If we found an existing login module with same class, we save it and delete later
                // delete later because it helps with formating to keep it while adding in the new one
                Logger.log('DEBUG',"Found login module : ${classname}")
            }else {
                Logger.log('DEBUG',"Did Not Find login module, ${classname}, in policy ${policy}")
            }
            //We assume there is an authentication node if there is a policy node...
            authenticationNode = loginmoduleParser.findNode("//security-domain[@name='${policy}']/authentication")            	
         } else {
            //No policy, go create one with an authentication node if not deleting
            if (deleted != "true")
            {
             loginmoduleParser.addTextNode("\n    ",policyNode)
             applicationpolicyNode = loginmoduleParser.addNode("security-domain","",policyNode)
        	 loginmoduleParser.addAttribute("name",policy,applicationpolicyNode)
        	 authenticationNode = loginmoduleParser.addNode("authentication","",applicationpolicyNode)
            }
         }

        
         if (deleted == "true")
         {
        	 Logger.log('DEBUG', "loginmodule - deleting login module $policy/$classname") 
         }
         else
         {
	         loginmoduleNode = loginmoduleParser.addNode("login-module","",authenticationNode)
	         loginmoduleParser.addAttribute("code",classname,loginmoduleNode)
	         loginmoduleParser.addAttribute("flag",flag,loginmoduleNode)
	          
	         for (option in options.tokenize(',')){
	             //Logger.log('DEBUG', "loginmodule - looping over options: option = $option")
	             def namevalue = option.split("=")
	             //Logger.log('DEBUG', "loginmodule - looping over options: namevalue = $namevalue")
	             def optionName = namevalue[0]
	             //Logger.log('DEBUG', "loginmodule - looping over options: optionName = $optionName")
	             def value = ""
	             def optionNode
	             def newValue
	             
	             if (namevalue.size()>2){
	                 value=namevalue[1]+"="+namevalue[2]
	                 //Logger.log('DEBUG', "loginmodule - looping over options: value = $value")
	             } else {
	                if (namevalue.size()>1){
	                 value=namevalue[1]
	                 //Logger.log('DEBUG', "loginmodule - looping over options: value = $value")
	             } else {
                      Logger.log('DEBUG', "Failure : option :$namevalue: is missing a required value, processing can not continue");
                      Logger.log('DEBUG', "Failure : dumping current thread stack for diagnostic purposes:");
                      Thread.dumpStack();
                      return 8;	             
	                 }
	             }
	             
	             def loggingValue
	             if (optionName == "password")
	                loggingValue = "xxxxxxxx"
	             else
	                loggingValue = value
	              
	             Logger.log('DEBUG', "loginmodule - processing options: optionName=$optionName value=$loggingValue");
	             if (! optionName.compareTo("managedConnectionFactoryName")){
	            	 if (! value.compareTo("JmsXA")){
	            		 newValue = "jboss.jca:service=TxCM,name=JmsXA"
	            	 }else{
	            		 newValue = "jboss.jca:name=$value,service=LocalTxCM"
						 if(classname.equals("org.picketbox.datasource.security.SecureIdentityLoginModule")){
						  if(!newValue.contains("jboss.jca:name=java:jboss/")){
							  newValue = newValue.replaceAll("jboss.jca:name=", "jboss.jca:name=java:jboss/")
						  }
						 }
	            	 }
	            	 optionNode = loginmoduleParser.addNode("module-option","",loginmoduleNode)
	            	 loginmoduleParser.addAttribute("name",optionName,optionNode)
					 loginmoduleParser.addAttribute("value",newValue,optionNode)
	             } else if (! optionName.compareTo("XAmanagedConnectionFactoryName") ){    
	                 newValue = "jboss.jca:name=$value,service=XATxCM"
					 if(classname.equals("org.picketbox.datasource.security.SecureIdentityLoginModule")){
						 if(!newValue.contains("jboss.jca:name=java:jboss/")){
							 newValue = newValue.replaceAll("jboss.jca:name=", "jboss.jca:name=java:jboss/")
						 }
 					 }

	                 optionNode = loginmoduleParser.addNode("module-option","",loginmoduleNode)
	                 loginmoduleParser.addAttribute("name","managedConnectionFactoryName",optionNode) 
					 loginmoduleParser.addAttribute("value",newValue,optionNode)
	             } else if (! optionName.compareTo("notset") ){    
                     // intentionally do nothing for this placeholder option
                 } else {
				     optionNode = loginmoduleParser.addNode("module-option","",loginmoduleNode)
	                 loginmoduleParser.addAttribute("name",optionName,optionNode)
					 loginmoduleParser.addAttribute("value",value,optionNode)
	             }
	         }
			 
			 //
			 //If this is a request to use the JBoss SecureIdentityLoginModule, we need to decode any SAS encoded password
			 // and then encode it with JBoss' encoding.
			 //
			 if(username!=null&&classname!=null)
			 if (( ! username.contains("not-set"))&&(classname.contains("org.picketbox.datasource.security.SecureIdentityLoginModule")) ){
				 def encodedPW
				 try{
                     String prompt = "\nEnter Password \n\tLoginModule=$classname \n\tusername=$username \n\tPassword="
				     password = com.sas.config.jboss.Utils.encodeDecodePassword(password, encoding, prompt)				 
					 Logger.log('DEBUG',"loginmodule - encoding for JBoss")
					 //encodedPW = org.jboss.resource.security.SecureIdentityLoginModule.encode(password);
					 encodedPW = org.picketbox.datasource.security.SecureIdentityLoginModule.encode(password);
					 
					 Logger.log('TRACE',"loginmodule - JBoss encoded password = $encodedPW")
				 }catch(Throwable e){
					 encodedPW = "PASSWORD"
					 Logger.log('INFO',"loginmodule - Unable to encode passwords, will use 'PASSWORD' instead.")
					 e.printStackTrace();
				 }
				 def optionNode = loginmoduleParser.addNode("module-option","",loginmoduleNode)
				 loginmoduleParser.addAttribute("name","password",optionNode)
				 loginmoduleParser.addAttribute("value",encodedPW,optionNode)
				 optionNode = loginmoduleParser.addNode("module-option","",loginmoduleNode)
				 loginmoduleParser.addAttribute("name","username",optionNode)
				 loginmoduleParser.addAttribute("value",username,optionNode)
			 }
			 
			 //
			 //If this is a request to use a SAS login module, we need to encode the password based on what 
			 // was passed in originally (encoding= setting)
			 //
			 if(username!=null&&classname!=null)
			 if (( ! username.contains("not-set"))&&(classname.contains("com.sas.services.security.login")) ){
				 def encodedPW
				 try{
					 String prompt = "\nEnter Password \n\tLoginModule=$classname \n\ttrusteduser=$username \n\tPassword="
					 encodedPW = com.sas.config.jboss.Utils.encodeDecodePassword(password, encoding, prompt)
				 }catch(Throwable e){
					 encodedPW = "PASSWORD"
					 Logger.log('INFO',"loginmodule - Unable to encode passwords, will use 'PASSWORD' instead.")
					 e.printStackTrace();
				 }
				 def optionNode = loginmoduleParser.addNode("module-option","",loginmoduleNode)
				 loginmoduleParser.addAttribute("name","trustedpw",optionNode)
				 loginmoduleParser.addAttribute("value",encodedPW,optionNode)
				 optionNode = loginmoduleParser.addNode("module-option","",loginmoduleNode)
				 loginmoduleParser.addAttribute("name","trusteduser",optionNode)
				 loginmoduleParser.addAttribute("value",username,optionNode)
			 }

         }
         if (oldModuleNode) {
             def textNode = oldModuleNode.getPreviousSibling();
             if (textNode.getNodeType() == org.w3c.dom.Node.TEXT_NODE) {
                  Logger.log('DEBUG', "loginmodule - removing previous whitespace text node...");
                  loginmoduleParser.removeNode(textNode);
              }
        	 loginmoduleParser.removeNode(oldModuleNode)
             def existingModuleNode = loginmoduleParser.findNode("//application-policy[@name='${policy}']/authentication/login-module")
             if (existingModuleNode == null)
             {
                 loginmoduleParser.removeNode(authenticationNode)
                 loginmoduleParser.removeNode(applicationpolicyNode)
             }

         }     	

             Logger.log('DEBUG', "loginmodule - Finished ... output loginmodule file...");
             loginmoduleParser.output(loginmoduleFile)
         }
         catch (Throwable e) {
             Logger.log('DEBUG', "Failure : " + e.toString());
             e.printStackTrace();
             return 8;
         }
         return 0;
    }
	
/*	int changeHost(String instance, Properties scriptProperties){
		try {

			Logger.log('DEBUG', "loginmodule - ChangeHost ${instance}")
			policy = scriptProperties.get("loginmodule."+instance+".policy")
			classname = scriptProperties.get("loginmodule."+instance+".classname")
			flag = scriptProperties.get("loginmodule."+instance+".flag")
			servername = scriptProperties.get("loginmodule."+instance+".servername")
			options = scriptProperties.get("loginmodule."+instance+".options")
			username = scriptProperties.get("loginmodule."+instance+".user")
			password = scriptProperties.get("loginmodule."+instance+".password")
			encoding = scriptProperties.get("loginmodule."+instance+".encoding")
			deleted = scriptProperties.get("loginmodule."+instance+".deleted")
			name = classname
			
			def jbossInstallDir = scriptProperties.'config.jboss.install.dir'
				
			def loginmoduleFile = new String("$jbossInstallDir/server/$servername/conf/login-config.xml")
			def loginmoduleParser = new Parser(loginmoduleFile)
	        def loginmoduleXpath="//application-policy[@name='${policy}']/authentication/login-module[@code='${classname}']"				 
            def loginmoduleNode=loginmoduleParser.findNode("//application-policy[@name='${policy}']/authentication/login-module[@code='${classname}']")
	        if (loginmoduleNode){
				for (option in options.tokenize(',')){
					//Logger.log('DEBUG', "loginmodule - looping over options: option = $option")
					def namevalue = option.split("=")
					//Logger.log('DEBUG', "loginmodule - looping over options: namevalue = $namevalue")
					def optionName = namevalue[0]
					//Logger.log('DEBUG', "loginmodule - looping over options: optionName = $optionName")
					def value = ""
					def optionNode
					def newValue
					
					if (namevalue.size()>2){
						value=namevalue[1]+"="+namevalue[2]
						//Logger.log('DEBUG', "loginmodule - looping over options: value = $value")
					} else {
					   if (namevalue.size()>1){
						value=namevalue[1]
						//Logger.log('DEBUG', "loginmodule - looping over options: value = $value")
					} else {
						 Logger.log('DEBUG', "Failure : option :$namevalue: is missing a required value, processing can not continue");
						 Logger.log('DEBUG', "Failure : dumping current thread stack for diagnostic purposes:");
						 Thread.dumpStack();
						 return 8;
						}
					}
					
					def loggingValue
					if (optionName == "host"){
	                      def moduleOptionNode=loginmoduleParser.findNode("$loginmoduleXpath/module-option[@name='host']")
						  if (moduleOptionNode){
				              Logger.log('DEBUG', "loginmodule - updating module-option name='host' with $value")
							  def newModuleOptionNode=loginmoduleParser.addNode("module-option",value, moduleOptionNode, "Sibling")
							  loginmoduleParser.addAttribute("name","host",newModuleOptionNode)
							  loginmoduleParser.removeNode(moduleOptionNode)
							  loginmoduleParser.output(loginmoduleFile)
						  } else {
						      Logger.log('DEBUG', "Failure : can't locate moduleOptionNode for $loginmoduleXpath/module-option[@name='host']")
						      Thread.dumpStack();
						 	  return 8;
						  }
					}
				}
			 } else {
			      Logger.log('DEBUG', "Failure : can't locate loginModule for $loginmoduleXpath")
			      Thread.dumpStack();
			 	  return 8;
			 }
		 }
		 catch (Throwable e) {
			 Logger.log('DEBUG', "Failure : " + e.toString());
			 e.printStackTrace();
			 return 8;
		 }
		 return 0;
	}*/

}
