// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved


package com.sas.config.jboss

import javax.xml.xpath.XPathFactory
import javax.xml.xpath.XPathConstants
import org.w3c.dom.Document
import org.w3c.dom.Node
import com.sas.config.jboss.*



class JBoss7Ports {

	String httpPort
	String httpsPort
	String jacorbPort
	String jacorbsslPort
	String jmxconnectorregistryPort
	String jmxconnectorserverPort
	String managementnativePort
	String managementhttpPort
	String messagingPort
	String messagingthroughoutPort
	String osgihttpPort
	String remotingPort
	String txnrecoveryenvironmentPort
	String txnstatusmanagerPort
    
	String server
	String name
	String portIncrement
	String bindingsFile
	Properties scriptProperties
	
	def defaultPortConfiguration = '''
   <socket-binding-group name="standard-sockets" default-interface="public"  >\n
        <socket-binding name="http" port="8080"/>\n
        <socket-binding name="https" port="8443"/>\n
        <socket-binding name="jacorb" port="3528"/>\n
        <socket-binding name="jacorb-ssl" port="3529"/>\n
        <socket-binding name="jmx-connector-registry" interface="management" port="1090"/>\n
        <socket-binding name="jmx-connector-server" interface="management" port="1091"/>\n
        <socket-binding name="management-native" interface="management" port="9999"/>\n
        <socket-binding name="management-http" interface="management" port="9990"/>\n
        <socket-binding name="messaging" port="5445"/>\n
        <socket-binding name="messaging-throughput" port="5455"/>\n
        <socket-binding name="osgi-http" interface="management" port="8090"/>\n
        <socket-binding name="remoting" port="4447"/>\n
        <socket-binding name="txn-recovery-environment" port="4712"/>\n
        <socket-binding name="txn-status-manager" port="4713"/>\n
        <outbound-socket-binding name="mail-smtp">\n
            <remote-destination host="localhost" port="25"/>\n
        </outbound-socket-binding>\n
    </socket-binding-group>\n

	'''

	JBoss7Ports (inServer, inName, inPortIncrement, inVersion, inScriptProp){
		server = inServer
		name = inName
		portIncrement = inPortIncrement
		version = inVersion
		scriptProperties = inScriptProp

		def installDir = scriptProperties.get("config.jboss.install.dir")
		bindingsFile = installDir+"/standalone/configuration/"+name+".xml"

		Logger.log('DEBUG', "JBossPorts - calling getProp(server."+server+".port.jndi,"+portIncrement+")")


		httpPort = getPort("server."+server+".port.http", portIncrement)
		httpsPort = getPort("server."+server+".port.https", portIncrement)
		jacorbPort = getPort("server."+server+".port.jacorb", portIncrement)
		jacorbsslPort = getPort("server."+server+".port.jacorbssl", portIncrement)
		jmxconnectorregistryPort = getPort("server."+server+".port.jmxconnectorregistry", portIncrement)
		jmxconnectorserverPort = getPort("server."+server+".port.jmxconnectorserver", portIncrement)
		managementnativePort = getPort("server."+server+".port.managementnative", portIncrement)
		managementhttpPort = getPort("server."+server+".port.managementhttp", portIncrement)
		messagingPort = getPort("server."+server+".port.messaging", portIncrement)
		messagingthroughoutPort = getPort("server."+server+".port.messagingthroughout", portIncrement)
		osgihttpPort = getPort("server."+server+".port.osgihttp", portIncrement)
		remotingPort = getPort("server."+server+".port.remoting", portIncrement)
		txnrecoveryenvironmentPort = getPort("server."+server+".port.txnrecoveryenvironment", portIncrement)
		txnstatusmanagerPort = getPort("server."+server+".port.txnstatusmanager", portIncrement)

		Logger.log('DEBUG', "Configuring " + server)
		Logger.log('DEBUG', "Ports - httpPort = " + httpPort)
		Logger.log('DEBUG', "        httpsPort = " + httpsPort)
		Logger.log('DEBUG', "        jacorbPort = " + jacorbPort)
		Logger.log('DEBUG', "        jacorbsslPort = " + jacorbsslPort)
		Logger.log('DEBUG', "        jmxconnectorregistryPort = " + jmxconnectorregistryPort)
		Logger.log('DEBUG', "        jmxconnectorserverPort = " + jmxconnectorserverPort)
		Logger.log('DEBUG', "        managementnativePort = " + managementnativePort)
		Logger.log('DEBUG', "        managementhttpPort = " + managementhttpPort)
		Logger.log('DEBUG', "        messagingPort = " + messagingPort)
		Logger.log('DEBUG', "        messagingthroughputPort = " + messagingthroughoutPort)
		Logger.log('DEBUG', "        osgihttpPort = " + osgihttpPort)
		Logger.log('DEBUG', "        remotingPort = " + remotingPort)
		Logger.log('DEBUG', "        txnrecoveryenvironmentPort = " + txnrecoveryenvironmentPort)
		Logger.log('DEBUG', "        txnstatusmanagerPort = " + txnstatusmanagerPort)

		def parser = new Parser(bindingsFile)
		def socketNode = parser.findNode("//socket-binding-group[@name='standard-sockets']")
		if(socketNode!=null){
		   parser.removeNode(socketNode)
		   parser.output(bindingsFile)
		}
		updateFile("</server>",defaultPortConfiguration+"</server>",bindingsFile)
		//def parser = new Parser()
		//parser.parse(bindingsFile)
		//def doc = parser.getDocument()

		//def elementRoot = doc.documentElement

		//def xpath = XPathFactory.newInstance().newXPath();



	}

	String getPort(prop, portIncrement){
		Logger.log('DEBUG', "getPort - prop = $prop")
		Integer port=0
		def tmpPort = scriptProperties.get(prop)
		def tmpProp = prop.substring("server..port.".length()+server.length())
		Logger.log('DEBUG', "getPort - tmpprop = $tmpProp")

		if (tmpPort == null){
			tmpPort = DefaultPorts47[tmpProp]
			tmpPort = (Integer.valueOf(tmpPort) + Integer.valueOf(portIncrement)).toString()
		}

		updatePort(prop, tmpPort)


		return tmpPort
	}

	String version

//	String updatePort(prop, port, file){
//
//		def propPort = scriptProperties.get(prop)
//		def newProp = prop.substring("server..port.".length()+server.length())
//		String defaultPort = DefaultPorts47[newProp]
//		if (defaultPort != port){
//			Logger.log('DEBUG', "updatePort - "+prop+" found in properties, updating "+defaultPort+" to "+port+" in "+file)
//			Map<String,String> replaceMap = new HashMap<String,String>()
//			replaceMap.put("port=\""+defaultPort+"\"", "port=\""+port+"\"")
//			Utils.replace (new File(file), replaceMap)
//		}
//
//		return port
//
//	}
	
	void updatePort(prop,port){
		def newProp = prop.substring("server..port.".length()+server.length())
		String defaultPort = DefaultPorts47[newProp]
		if (defaultPort != port){
			Logger.log('DEBUG', "updatePort - "+prop+" found in properties, updating "+defaultPort+" to "+port+" in config file")
			//Map<String,String> replaceMap = new HashMap<String,String>()
			//replaceMap.put("port=\""+defaultPort+"\"", "port=\""+port+"\"")
			//Utils.replace (new File(file), replaceMap)
			defaultPortConfiguration = defaultPortConfiguration.replaceAll(defaultPort, port)
		}

	}
	def updateFile(oldText,newText,file){
		Map<String,String> replaceMap = new HashMap<String,String>()
		replaceMap.put(oldText, newText)
		Utils.replace (new File(file), replaceMap)
   
    }
}
public class DefaultPorts47 {
	static final String http="8080"
	static final String https="8443"
	static final String jacorb="3528"
	static final String jacorbssl="3529"
	static final String jmxconnectorregistry="1090"
	static final String jmxconnectorserver="1091"
	static final String managementnative="9999"
	static final String managementhttp="9990"
	static final String messaging="5445"
	static final String messagingthroughout="5455"
	static final String osgihttp="8090"
	static final String remoting="4447"
	static final String txnrecoveryenvironment="4712"
	static final String txnstatusmanager="4713"


	DefaultPorts47(){}

}