// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved


import groovy.util.CliBuilder

import java.io.File
import java.text.*

import javax.xml.parsers.DocumentBuilderFactory
import javax.xml.parsers.DocumentBuilder
import javax.xml.transform.dom.DOMSource
import javax.xml.transform.stream.StreamResult
import javax.xml.transform.TransformerFactory
import javax.xml.transform.OutputKeys
import javax.xml.xpath.XPathFactory
import javax.xml.xpath.XPathConstants

import com.sas.config.jboss.Server
import com.sas.config.jboss.Connectionfactory
import com.sas.config.jboss.Queue
import com.sas.config.jboss.Topic
import com.sas.config.jboss.Datasource
import com.sas.config.jboss.Mailsession
import com.sas.config.jboss.Loginmodule
import com.sas.config.jboss.Externalcontext
import com.sas.config.jboss.JDBCProvider
import com.sas.config.jboss.Application
import com.sas.config.jboss.ResourceInterface
import com.sas.config.jboss.Logger
import com.sas.config.jboss.*

import org.codehaus.groovy.tools.xml.DomToGroovy
import org.w3c.dom.Document
import org.w3c.dom.Node


def int testResult (ResourceInterface ri, int rc) {
    switch (rc) {
      case 8 : 
           Logger.log('DEBUG', "$ri.resourceType failed to configure name=$ri.name rc=$rc"); 
           return 8;
      case 0 :
      default :
           // Logger.log('DEBUG', "$ri.resourceType succeeded to configure name=$ri.name rc=$rc"); 
           break;
     }
     return rc;
}

/*
 * 
 * START OF MAIN 
 * 
 */

groovy.util.CliBuilder cli = new CliBuilder(usage: 'jbossScripting -[acdehlmqst] -[n resourceName] [propertyFile]') 
cli.with {
   h longOpt: 'help', 'Show usage information'
   a longOpt: 'applications', 'Configure all Applications'
   c longOpt: 'connectionFactories', 'Configure all Connection Factories'
   d longOpt: 'datasources', 'Configure all Datasources'
   e longOpt: 'externalContexts', 'Configure all externalContexts'
   j longOpt: 'jdbcprovider', 'Configure all jdbcproviders'
   l longOpt: 'loginmodules', 'Configure all Login Modules'
   m longOpt: 'mailsessions', 'Configure all Mail Sessions'
   q longOpt: 'queues', 'Configure all Queues'
   s longOpt: 'servers', 'Configure all Servers'
   t longOpt: 'topics', 'Configure all Topics'
   n longOpt: 'resourcename', args: 1, argName: 'name', 'Configure one named Resource'
   _ longOpt: 'changeHost', 'Run ChangeHost methods instead of Configure Methods'
   _ longOpt: 'unconfigure', 'Run unconfigure methods instead of Configure Methods'
}

groovy.util.OptionAccessor options = cli.parse(args)     
if (!options || options.h) {     
   cli.usage()
   return
} 
  
def configureApplications=false
def configureConnectionFactories=false
def configureDatasources=false
def configureExternalContexts=false
def configureJdbcProviders=false
def configureLoginModules=false
def configureMailSessions=false
def configureQueues=false
def configureServers=false
def configureTopics=false
def confMethod="configure"

if (options.a || options.applications) {
   configureApplications=true
}
if (options.c || options.connectionFactories) {
   configureConnectionFactories=true
}
if (options.d || options.datasources) {
   configureDatasources=true
}
if (options.e || options.externalContexts) {
   configureExternalContexts=true
}
if (options.j || options.jdbcproviders) {
   configureJdbcProviders=true
}
if (options.l || options.loginModules) {
   configureLoginModules=true
}
if (options.m || options.mailSessions) {
   configureMailSessions=true
}
if (options.q || options.queues) {
   configureQueues=true
}
if (options.s || options.servers) {
   configureServers=true
}
if (options.t || options.topics) {
   configureTopics=true
}
if (options.changeHost) {
   confMethod="changeHost"
}
if (options.unconfigure) {
   confMethod="unconfigure"
}

if (!(options.a || options.c || options.d || options.e || options.j || options.l || options.m || options.q || options.s || options.t)){
    configureApplications=true
    configureConnectionFactories=true
    configureDatasources=true
    configureExternalContexts=true
    configureJdbcProviders=true
    configureLoginModules=true
    configureMailSessions=true
    configureQueues=true
    configureServers=true
    configureTopics=true
}

File current = new File(".")
def propertiesFile=current.getCanonicalPath() + current.separator + "jbossScripting.properties"

if (options.arguments().size()>0) {
  tmpFileNameString=options.arguments()[0]
  File tmpPropFile = new File(tmpFileNameString)
  if (tmpPropFile.exists()){
	propertiesFile = tmpFileNameString;
  } else {
    println("jbossScripting: failed to locate property file ($tmpFileNameString), using default $propertiesFile")
    cli.usage()
    return(8);
  } 
}

// Load Scripting Properties
scriptProperties = new Properties()
scriptProperties.load(new FileInputStream(propertiesFile))

def parentDir = (new File(propertiesFile)).getParent();
if (parentDir == null){
   parentDir = "."
}
passwordFile = parentDir + "/jbossScriptingCredentials.properties"
scriptProperties.load(new FileInputStream(passwordFile))

String servers = scriptProperties.'server'
String connectionfactories = scriptProperties.'connectionfactory'
String queues = scriptProperties.'queue'
String topics = scriptProperties.'topic'
String datasources = scriptProperties.'datasource'
String mailsessions = scriptProperties.'mailsession'
String loginmodules = scriptProperties.'loginmodule'
String applications = scriptProperties.'application'
String externalcontexts = scriptProperties.'externalcontext'
String jdbcproviders = scriptProperties.'jdbcprovider'
String listseparator = scriptProperties.'listseparator'
String jbossInstallDir = scriptProperties.'config.jboss.install.dir'

Logger.logLevel = new String(scriptProperties.'config.loglevel')

if (listseparator==null) listseparator = / /

Logger.log('DEBUG', "jbossScripting: properties file contains:")
Logger.log('DEBUG', "jbossScripting:       jbossInstallDir = " + jbossInstallDir)
Logger.log('DEBUG', "jbossScripting:       servers = " + servers)
Logger.log('DEBUG', "jbossScripting:       connectionfactories = " + connectionfactories)
Logger.log('DEBUG', "jbossScripting:       queues = " + queues)
Logger.log('DEBUG', "jbossScripting:       topics = " + topics)
Logger.log('DEBUG', "jbossScripting:       datasources = " + datasources)
Logger.log('DEBUG', "jbossScripting:       mailsessions = " + mailsessions)
Logger.log('DEBUG', "jbossScripting:       loginmodules = " + loginmodules)
Logger.log('DEBUG', "jbossScripting:       applications = " + applications)
Logger.log('DEBUG', "jbossScripting:       externalcontext = " + externalcontexts)
Logger.log('DEBUG', "jbossScripting:       jdbcprovider = " + jdbcproviders)
Logger.log('DEBUG', "jbossScripting:       listseparator = " + listseparator)

def serverList=[]
def connectionfactoryList=[]
def queueList=[]
def topicList=[]
def datasourceList=[]
def mailsessionList=[]
def loginmoduleList=[]
def applicationList=[]
def externalcontextList=[]
def jdbcproviderList=[]
def config=null
def rc=8

String[] getList(String listStr, groovy.util.OptionAccessor myOptions,groovy.util.CliBuilder myCli,String listseparator){
  def list=[]
  Boolean single = myOptions.n
  String resourcename = myOptions.resourcename
  if (single) {
     if (listStr.contains(resourcename)) {
	Logger.log('DEBUG', "jbossScripting: adding " + resourcename + " to config list")
        list.add(resourcename)
     } else {
	Logger.log('DEBUG', "jbossScripting:  can't locate resource " + resourcename)
	myCli.usage()
     }
     
  } else {
     list = listStr.split(listseparator)
     Logger.log('DEBUG', "jbossScripting: adding " + list + " to config list")
  }
  return list
}

if (servers!=null && servers!="" && configureServers) serverList = getList(servers,options,cli,listseparator)
if (connectionfactories!=null && servers!="" && configureConnectionFactories) connectionfactoryList= getList(connectionfactories,options,cli,listseparator)
if (queues!=null && servers!="" && configureQueues) queueList= getList(queues,options,cli,listseparator)
if (topics!=null && servers!="" && configureTopics) topicList= getList(topics,options,cli,listseparator)
if (datasources!=null && servers!="" && configureDatasources) datasourceList= getList(datasources,options,cli,listseparator)
if (mailsessions!=null && servers!="" && configureMailSessions) mailsessionList= getList(mailsessions,options,cli,listseparator)
if (loginmodules!=null && servers!="" && configureLoginModules) loginmoduleList= getList(loginmodules,options,cli,listseparator)
if (applications!=null && servers!="" && configureApplications) applicationList= getList(applications,options,cli,listseparator)
if (externalcontexts!=null && servers!="" && configureExternalContexts) externalcontextList= getList(externalcontexts,options,cli,listseparator)
if (jdbcproviders!=null && servers!="" && configureJdbcProviders) jdbcproviderList= getList(jdbcproviders,options,cli,listseparator)

//Here are the main loops for each component
serverList.each { 
    Logger.log('DEBUG',"jbossScripting: processing server $it")
    config = new JBossServer7()
    rc = config."$confMethod"(it,scriptProperties)
    testResult(config, rc);
}

connectionfactoryList.each {  
    Logger.log('DEBUG',"jbossScripting: processing connectionfactory $it")
    config = new Connectionfactory47()
    rc = config."$confMethod"(it,scriptProperties)
    testResult(config, rc);
}

queueList.each {  
    Logger.log('DEBUG',"jbossScripting: processing queue $it")
    config = new Queue47()
    rc = config."$confMethod"(it,scriptProperties)
    testResult(config, rc);
}

topicList.each {  
    Logger.log('DEBUG',"jbossScripting: processing topic $it")
    config = new Topic47()
    rc = config."$confMethod"(it,scriptProperties)
    testResult(config, rc);
}

datasourceList.each {
    Logger.log('DEBUG',"jbossScripting: processing datasource $it")
    config = new Datasource47()
    rc = config."$confMethod"(it,scriptProperties)
    testResult(config, rc);
}

mailsessionList.each{
    Logger.log('DEBUG',"jbossScripting: processing mailsession $it")
    config = new Mailsession47()  
    rc = config."$confMethod"(it,scriptProperties)
    testResult(config, rc);
}

loginmoduleList.each{
    Logger.log('DEBUG',"jbossScripting: processing loginmodule $it")
    config = new Loginmodule47()    
    rc = config."$confMethod"(it,scriptProperties)
    testResult(config, rc);
}

externalcontextList.each{
    //Logger.log('DEBUG',"jbossScripting: processing externalcontext $it")
    //config = new Externalcontext()    
    //rc = config."$confMethod"(it,scriptProperties)
    //testResult(config, rc);
}

jdbcproviderList.each{
   /* Logger.log('DEBUG',"jbossScripting: checking on jdbcprovider $it")
    Logger.log('DEBUG',"jbossScripting: options.n = $options.n")
    def propName="jdbcprovider."+it+".name"
    Logger.log('DEBUG',"jbossScripting: propName = $propName")
    if ((options.n && (scriptProperties.getProperty(propName) == options.n)) || !options.n)
    {
        Logger.log('DEBUG',"jbossScripting: processing jdbcprovider $it")
        config = new JDBCProvider()
        Logger.log('DEBUG', "jbossScripting - calling JDBCProvider.$confMethod($it,scriptProperties)");
                
        rc = config."$confMethod"(it,scriptProperties)
        testResult(config, rc);
    }*/
}

applicationList.each{
    Logger.log('DEBUG',"jbossScripting: processing application $it")
    config = new Application47()
    rc = config."$confMethod"(it,scriptProperties)
    testResult(config, rc);
}

/*
 * 
 * END OF MAIN 
 * 
 */
 

